@@ -32,6 +32,7 @@
 #include <glade/glade.h>
 
 #include "dlg-save-image.h"
+#include "glib-utils.h"
 #include "file-utils.h"
 #include "gtk-utils.h"
 #include "gconf-utils.h"
@@ -87,76 +88,57 @@ file_save_cancel_cb (GtkDialog *file_sel,
 
 
 static gboolean
-save_image (GtkWindow       *parent,
-     	    const char      *filename,
-	    const char      *mime_type,
-	    GdkPixbuf       *pixbuf,
-	    SaveImageData   *data,
-	    GtkDialog       *file_sel)
+save_image (GtkWindow     *parent,
+     	    FileData      *file,
+	    GdkPixbuf     *pixbuf,
+	    SaveImageData *data,
+	    GtkDialog     *file_sel)
 {
-	gboolean  file_exists;
-	gboolean  image_saved = FALSE;
+	gboolean image_saved = FALSE;
 
-	if (filename == NULL)
+	if (file == NULL)
 		return FALSE;
 
-	file_exists = path_is_file (filename);
-
-	if ((file_sel != NULL) && file_exists) {
-		char      *message;
-		GtkWidget *d;
-		int        r;
-
-		message = g_strdup_printf (_("An image named \"%s\" is already present. Do you want to overwrite it?"), file_name_from_path (filename));
-		d = _gtk_yesno_dialog_new (parent,
-					   GTK_DIALOG_MODAL,
-					   message,
-					   GTK_STOCK_NO,
-					   GTK_STOCK_YES);
-		g_free (message);
-
-		r = gtk_dialog_run (GTK_DIALOG (d));
-		gtk_widget_destroy (d);
-
-		if (r != GTK_RESPONSE_YES)
-			return FALSE;
-	}
-
 	if (file_sel != NULL)
 		gtk_widget_hide (GTK_WIDGET (file_sel));
 
-	if ((mime_type != NULL)
-	    && is_mime_type_writable (mime_type)) {
-		GError      *error = NULL;
-		const char  *image_type = mime_type + strlen("image/");
-		char       **keys = NULL;
-		char       **values = NULL;
+	if ((file->mime_type != NULL) && is_mime_type_writable (file->mime_type)) {
+		GError  *error = NULL;
+		char    *image_type;
+		char   **keys = NULL;
+		char   **values = NULL;
 
-		if (dlg_save_options (parent,
-				      image_type,
-				      &keys,
-				      &values)) {
-			if (! _gdk_pixbuf_savev (pixbuf,
-						 filename,
-						 image_type,
-						 keys, values,
-						 &error))
-				_gtk_error_dialog_from_gerror_run (parent, &error);
-			else
+		image_type = g_strdup (file->mime_type + strlen("image/"));
+
+		if (dlg_save_options (parent, image_type, &keys, &values)) {
+			char *local_file;
+			
+			local_file = get_cache_filename_from_uri (file->path);
+			if (_gdk_pixbuf_savev (pixbuf,
+					       local_file,
+					       image_type,
+					       keys, values,
+					       &error))
 				image_saved = TRUE;
+			else
+				_gtk_error_dialog_from_gerror_run (parent, &error);
+				
+			g_free (local_file);
 		}
 
+		g_free (image_type);
 		g_strfreev (keys);
 		g_strfreev (values);
-	} else
+	} 
+	else
 		_gtk_error_dialog_run (parent,
 				       _("Image type not supported: %s"),
-				       mime_type);
+				       file->mime_type);
 
 	if (data->done_func != NULL) {
 		if (! image_saved)
-			filename = NULL;
-		(*data->done_func) (filename, data->done_data);
+			file = NULL;
+		(*data->done_func) (file, data->done_data);
 	}
 
 	if (file_sel != NULL)
@@ -176,7 +158,7 @@ file_save_ok_cb (GtkDialog *file_sel,
 	GtkWindow     *parent;
 	GtkWidget     *opt_menu;
 	GdkPixbuf     *pixbuf;
-	char          *filename = NULL;
+	FileData      *file = NULL;
 	const char    *mime_type = NULL;
 	int            idx;
 	SaveImageData *data;
@@ -184,18 +166,19 @@ file_save_ok_cb (GtkDialog *file_sel,
 	parent = g_object_get_data (G_OBJECT (file_sel), "parent_window");
 	pixbuf = g_object_get_data (G_OBJECT (file_sel), "pixbuf");
 	data = g_object_get_data (G_OBJECT (file_sel), "data");
-	filename = g_strdup (gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel)));
+
+	file = file_data_new (gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (file_sel)), NULL);
 
 	opt_menu = g_object_get_data (G_OBJECT (file_sel), "opt_menu");
 	idx = gtk_option_menu_get_history (GTK_OPTION_MENU (opt_menu));
 	if (idx == IMAGE_TYPE_AUTOMATIC)
-		mime_type = gnome_vfs_get_file_mime_type (filename, NULL, FALSE);
+		mime_type = get_file_mime_type (file->path, FALSE);
 	else
 		mime_type = mime_types [idx - 2];
-
-	save_image (parent, filename, mime_type, pixbuf, data, file_sel);
-
-	g_free (filename);
+	file->mime_type = get_static_string (mime_type);
+	
+	save_image (parent, file, pixbuf, data, file_sel);
+	g_free (file);
 }
 
 
@@ -260,10 +243,8 @@ dlg_save_image_as (GtkWindow       *parent,
 					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					       GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 					       NULL);
-
-	 /* Permit VFS URIs */
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (file_sel), FALSE);
-
+	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (file_sel), TRUE);
 	gtk_dialog_set_default_response (GTK_DIALOG (file_sel), GTK_RESPONSE_ACCEPT);
 
 	/**/
@@ -292,7 +273,7 @@ dlg_save_image_as (GtkWindow       *parent,
 	if (uri != NULL)
 		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (file_sel), uri);
 	else
-		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (file_sel), get_home_uri ());
+		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (file_sel), get_home_uri ());
 
 	g_object_ref (pixbuf);
 
@@ -324,32 +305,26 @@ dlg_save_image_as (GtkWindow       *parent,
 }
 
 
-
 void
 dlg_save_image (GtkWindow       *parent,
-		const char      *filename,
+		FileData        *file,
 		GdkPixbuf       *pixbuf,
 		ImageSavedFunc   done_func,
 		gpointer         done_data)
 {
 	SaveImageData *data;
-	const char    *mime_type = NULL;
 
-	if (filename == NULL)
+	if (file == NULL)
 		return;
 
 	data = g_new0 (SaveImageData, 1);
 	data->done_func = done_func;
 	data->done_data = done_data;
 
-	mime_type = gnome_vfs_get_file_mime_type (filename, NULL, FALSE);
-
-	save_image (parent, filename, mime_type, pixbuf, data, NULL);
+	save_image (parent, file, pixbuf, data, NULL);
 }
 
 
-
-
 #define GLADE_FILE "gthumb_convert.glade"
 
 
