@@ -332,19 +332,18 @@ debug (const char *file,
        const char *function,
        const char *format, ...)
 {
-	static gboolean first_time = 0;
-	static gboolean print_debug_info = 0;
+	static   gboolean first_time = 0;
+	static   gboolean print_debug_info = 0;
 	va_list  args;
 	char    *str;
 
-	if (!first_time) {
+	if (! first_time) {
 		first_time = 1;
-		if(g_getenv("GTHUMB_DEBUG")) {
+		if (g_getenv ("GTHUMB_DEBUG")) 
 			print_debug_info = 1;
-		} 
 	}
 
-	if (!print_debug_info)
+	if (! print_debug_info)
 		return;
 
 	g_return_if_fail (format != NULL);
@@ -398,7 +397,7 @@ get_file_list_from_url_list (char *url_list)
 
 
 int
-strcmp_null_tollerant (const char *s1, const char *s2)
+strcmp_null_tolerant (const char *s1, const char *s2)
 {
         if ((s1 == NULL) && (s2 == NULL))
                 return 0;
@@ -512,3 +511,28 @@ idle_call_exec (IdleCall *call,
 		return 0;
 	}
 }
+
+
+GHashTable *static_strings = NULL;
+
+
+const char *
+get_static_string (const char *s)
+{
+	const char *result;
+
+	if (s == NULL)
+		return NULL;
+
+	if (static_strings == NULL)
+		static_strings = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+
+	if (! g_hash_table_lookup_extended (static_strings, s, (gpointer*) &result, NULL)) {
+		result = g_strdup (s);
+		g_hash_table_insert (static_strings,
+				     (gpointer) result,
+				     GINT_TO_POINTER (1));
+	}
+
+	return result;
+}
