@@ -22,7 +22,6 @@
 
 #include <config.h>
 #include <unistd.h>
-#include <libgnome/libgnome.h>
 
 #include "catalog.h"
 #include "file-utils.h"
@@ -52,7 +51,7 @@ ensure_directories_exist (void)
         /* before the gconf port this was a file, now it's folder. */
         path = get_home_relative_dir (RC_DIR);
         if (path_is_file (path))
-                unlink (path);
+                file_unlink (path);
 	g_free (path);
 
 	path = get_home_relative_dir (RC_CATALOG_DIR);
@@ -62,6 +61,10 @@ ensure_directories_exist (void)
 	path = get_home_relative_dir (RC_COMMENTS_DIR);
 	ensure_dir_exists (path, 0700);
 	g_free (path);
+
+	path = get_home_relative_dir (RC_REMOTE_CACHE_DIR);
+        ensure_dir_exists (path, 0700);
+        g_free (path);
 }
 
 
@@ -151,7 +154,7 @@ gthumb_init ()
 		migrate_to_new_directories ();
 
 	ensure_directories_exist ();
-
+		
 	eel_gconf_monitor_add ("/apps/gthumb/browser");
 	eel_gconf_monitor_add ("/apps/gthumb/ui");
 	eel_gconf_monitor_add ("/apps/gthumb/viewer");
@@ -176,4 +179,5 @@ gthumb_release ()
 	eel_gconf_monitor_remove ("/apps/gthumb/viewer");
 
 	preferences_release ();
+	free_cache ();
 }
