@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001, 2003, 2004, 2005 Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003, 2004, 2005, 2007 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -76,7 +76,7 @@ typedef struct {
 	GList        *current_image;
 	ImageLoader  *loader;
 	GdkPixbuf    *original_preview;
-	GthTransform	transform;
+	GthTransform  transform;
 } DialogData;
 
 
@@ -91,7 +91,7 @@ dialog_data_free (DialogData *data)
 
 	g_idle_add ((GSourceFunc)all_windows_add_monitor, NULL);
 
-	path_list_free (data->file_list);
+	file_data_list_free (data->file_list);
 	if (data->loader != NULL)
 		g_object_unref (data->loader);
 	if (data->gui != NULL)
@@ -154,15 +154,16 @@ image_loader_done_cb (ImageLoader  *il,
 	GdkPixbuf *pixbuf;
 
 	pixbuf = image_loader_get_pixbuf (il);
-	if (pixbuf != NULL) {
-		if (data->original_preview != NULL)
-			g_object_unref (data->original_preview);
-		data->original_preview = _gdk_pixbuf_scale_keep_aspect_ratio (pixbuf, PREVIEW_SIZE, PREVIEW_SIZE);
-		gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), data->original_preview);
+	if (pixbuf == NULL) 
+		return;
+		
+	if (data->original_preview != NULL)
+		g_object_unref (data->original_preview);
+	data->original_preview = _gdk_pixbuf_scale_keep_aspect_ratio (pixbuf, PREVIEW_SIZE, PREVIEW_SIZE);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), data->original_preview);
 
-		gtk_widget_set_sensitive (data->j_button_vbox, TRUE);
-		gtk_widget_set_sensitive (data->j_revert_button, TRUE);
-	}
+	gtk_widget_set_sensitive (data->j_button_vbox, TRUE);
+	gtk_widget_set_sensitive (data->j_revert_button, TRUE);
 }
 
 
@@ -178,8 +179,6 @@ image_loader_error_cb (ImageLoader  *il,
 static void
 load_current_image (DialogData *data)
 {
-	char *path;
-
 	if (data->current_image == NULL) {
 		gtk_widget_destroy (data->dialog);
 		return;
@@ -188,8 +187,7 @@ load_current_image (DialogData *data)
 	gtk_widget_set_sensitive (data->j_button_vbox, FALSE);
 	gtk_widget_set_sensitive (data->j_revert_button, FALSE);
 
-	path = data->current_image->data;
-	image_loader_set_path (data->loader, path);
+	image_loader_set_file (data->loader, (FileData*) data->current_image->data);
 	image_loader_start (data->loader);
 
 	data->transform = GTH_TRANSFORM_NONE;
@@ -214,6 +212,20 @@ load_next_image (DialogData *data)
 }
 
 
+/* -- apply_transformation -- */
+
+
+typedef struct {
+	GtkWidget        *parent_window;
+	DialogData       *data;
+	GList            *current_image;
+	GnomeVFSFileInfo *info;
+	gboolean          notify_soon;
+	CopyDoneFunc      done_func;
+	gpointer          done_data;
+} ApplyTransformData;
+
+
 static void
 notify_file_changed (DialogData *data,
 		     const char *filename,
@@ -223,165 +235,230 @@ notify_file_changed (DialogData *data,
 		GList *list = g_list_prepend (NULL, (char*) filename);
 		all_windows_notify_files_changed (list);
 		g_list_free (list);
-	} else
+	} 
+	else
 		data->files_changed_list = g_list_prepend (data->files_changed_list, g_strdup (filename));
 }
 
 
-#define GTK_RESPONSE_TRIM 1
-static gint jpeg_mcu_dialog (GtkWindow *parent)
+static void
+apply_transformation_done (const char     *uri,
+			   GnomeVFSResult  result,
+		           gpointer        callback_data)
 {
- 	GtkWidget *d =  _gtk_message_dialog_new (parent,
-		GTK_DIALOG_MODAL,
-		GTK_STOCK_DIALOG_WARNING,
-		_("This transformation may introduce small image distortions along "
-		"one or more edges, because the image dimensions are not multiples of 8.\n\nThe distortion "
-		"is reversible, however. If the resulting image is unacceptable, simply apply the reverse "
-		"transformation to return to the original image.\n\ngThumb can also discard (or trim) any "
-		"untransformable edge pixels. For practical use, this mode gives the best looking results, "
-		"but the transformation is not strictly lossless anymore.\n\nTo avoid this problem in the "
-		"future, consider disabling the \"Apply physical transform\" option in the rotation dialog."),
-		NULL,
-		_("Trim"), GTK_RESPONSE_TRIM,
-		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_OK, GTK_RESPONSE_OK,
-		NULL);
+	ApplyTransformData *at_data = callback_data;
+	FileData           *file = at_data->current_image->data;
+		
+	if (result == GNOME_VFS_OK) {
+		if (at_data->info != NULL)
+			gnome_vfs_set_file_info (file->path, at_data->info, GNOME_VFS_SET_FILE_INFO_PERMISSIONS | GNOME_VFS_SET_FILE_INFO_OWNER);
+		notify_file_changed (at_data->data, file->path, at_data->notify_soon);
+	}
+	else 
+		_gtk_error_dialog_run (GTK_WINDOW (at_data->data->dialog), _("Could not move temporary file to remote location. Check remote permissions."));
+	
+	if (at_data->done_func)
+		(at_data->done_func) (uri, result, at_data->done_data);
 
-	gint result = gtk_dialog_run (GTK_DIALOG (d));
-
- 	gtk_widget_destroy (d);
-
-	if (result == GTK_RESPONSE_DELETE_EVENT)
-		return GTK_RESPONSE_CANCEL;
-	else
- 		return result;
+	if (at_data->info != NULL)
+		gnome_vfs_file_info_unref (at_data->info);
+	g_free (at_data);
 }
 
 
 static void
-apply_transformation (DialogData *data,
-		      GList      *current_image,
-		      gboolean    notify_soon)
+apply_transformation__trim_response (JpegMcuAction action,
+				     gpointer      callback_data)
 {
-	char             *path = current_image->data;
-	char             *dir;
-	GnomeVFSFileInfo  info;
-	GtkWindow  	 *window = GTK_WINDOW (data->dialog);
-	GthTransform	  required_transform;
+	ApplyTransformData *at_data = callback_data;
+	FileData           *file = at_data->current_image->data;
+	char		   *local_file = NULL;
+	ExifShort           image_orientation;
+	GthTransform	    required_transform;
 
-	/* Check directory permissions. */
+	local_file = get_cache_filename_from_uri (file->path);
+	image_orientation = get_exif_tag_short (local_file, EXIF_TAG_ORIENTATION);
+	required_transform = get_next_transformation (image_orientation, at_data->data->transform);
+	
+	apply_transformation_jpeg (file, required_transform, action, NULL);
+	
+	g_free (local_file);
+	
+	update_file_from_cache (file, apply_transformation_done, at_data);
+}
 
-	dir = remove_level_from_path (path);
-	if (! check_permissions (dir, R_OK | W_OK | X_OK)) {
-		char *utf8_path = g_filename_display_name (dir);
-		_gtk_error_dialog_run (GTK_WINDOW (data->dialog),
-				       _("You don't have the right permissions to create images in the folder \"%s\""),
-				       utf8_path);
-		g_free (utf8_path);
-		g_free (dir);
-		return;
-	}
-	g_free (dir);
 
-	gnome_vfs_get_file_info (path, &info, GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS|GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-
-	gboolean jpeg = image_is_jpeg (path);
-	ExifShort orientation = get_exif_tag_short(path, EXIF_TAG_ORIENTATION);
-	required_transform = get_next_transformation (orientation, data->transform);
-
-	if ((jpeg && orientation) &&
-	    !eel_gconf_get_boolean (PREF_ROTATE_RESET_EXIF_ORIENTATION, TRUE)) {
-		// Adjust Exif orientation tag.
-		write_orientation_field (path, required_transform);
-	} else if (jpeg) {
- 		// Lossless jpeg transform.
-		gint width, height;
-		gdk_pixbuf_get_file_info (get_file_path_from_uri (path), &width, &height);
-		if (!jtransform_perfect_transform (width, height, 8, 8, required_transform)) {
- 			// Image dimensions are not multiples of the jpeg minimal coding unit (mcu).
- 			// Warn about possible image distortions along one or more edges.
- 			gint result = jpeg_mcu_dialog (window);
-			if (result != GTK_RESPONSE_CANCEL) {
-				gboolean trim = (result == GTK_RESPONSE_TRIM);
-				apply_transformation_jpeg (window, path, required_transform, trim);
+static void
+apply_transformation__step2 (const char     *uri,
+			     GnomeVFSResult  result,
+		             gpointer        callback_data)
+{
+	ApplyTransformData *at_data = callback_data;
+	FileData           *file = at_data->current_image->data;
+	char		   *local_file = NULL;
+	ExifShort           image_orientation;
+	GthTransform	    required_transform;
+	gboolean            go_on = TRUE;
+	
+	local_file = get_cache_filename_from_uri (file->path);
+	image_orientation = get_exif_tag_short (local_file, EXIF_TAG_ORIENTATION);
+	required_transform = get_next_transformation (image_orientation, at_data->data->transform);
+	if (mime_type_is (file->mime_type, "image/jpeg")) {
+		if (image_orientation && ! eel_gconf_get_boolean (PREF_ROTATE_RESET_EXIF_ORIENTATION, TRUE)) 
+			/* Adjust Exif orientation tag. */
+			write_orientation_field (local_file, required_transform);
+		else { 
+			GError *error = NULL;
+ 			
+ 			/* Lossless jpeg transform. */
+ 			
+			if (! apply_transformation_jpeg (file, 
+							 required_transform, 
+							 JPEG_MCU_ACTION_ABORT, 
+							 &error)) {
+				if (error->code == JPEGTRAN_ERROR_MCU) {
+					ask_whether_to_trim (GTK_WINDOW (at_data->data->window), file, apply_transformation__trim_response, at_data);
+					go_on = FALSE;
+				}
 			}
- 		} else {
-			apply_transformation_jpeg (window, path, required_transform, FALSE);
 		}
-	} else {
-		// Generic image transform.
-		apply_transformation_generic (window, path, required_transform);
+	}
+	else 
+		/* Generic image transform. */
+		apply_transformation_generic (file, required_transform, NULL);
+	g_free (local_file);
+	
+	if (go_on)
+		update_file_from_cache (file, apply_transformation_done, at_data);
+}
+
+
+static void
+apply_transformation (GtkWidget    *parent_window,
+		      DialogData   *data,
+		      GList        *current_image,
+		      gboolean      notify_soon,
+		      CopyDoneFunc  done_func,
+		      gpointer      done_data)
+{
+	FileData           *file = current_image->data;
+	ApplyTransformData *at_data;
+
+	at_data = g_new0 (ApplyTransformData, 1);
+	at_data->parent_window = parent_window;
+	at_data->data = data;
+	at_data->current_image = current_image;
+	at_data->notify_soon = notify_soon;
+	at_data->done_func = done_func;
+	at_data->done_data = done_data;
+	at_data->info = gnome_vfs_file_info_new ();
+	if (gnome_vfs_get_file_info (file->path, at_data->info, GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS|GNOME_VFS_FILE_INFO_FOLLOW_LINKS) != GNOME_VFS_OK) {
+		gnome_vfs_file_info_unref (at_data->info);
+		at_data->info = NULL;
 	}
 
-	gnome_vfs_set_file_info (path, &info, GNOME_VFS_SET_FILE_INFO_PERMISSIONS|GNOME_VFS_SET_FILE_INFO_OWNER);
+	copy_remote_file_to_cache (file, apply_transformation__step2, at_data);
+}
 
-	notify_file_changed (data, path, notify_soon);
+
+typedef struct {
+	DialogData *data;
+	GladeXML   *gui;
+	GtkWidget  *dialog;
+	GtkWidget  *label;
+	GtkWidget  *bar;
+	GList      *scan;
+	int         i, n;
+} BatchTransformation;
+
+
+static void apply_transformation_to_all__apply_to_current (BatchTransformation *);
+
+
+static void
+apply_transformation_to_all_continue (const char     *uri,
+				      GnomeVFSResult  result,
+				      gpointer        data)
+{
+	BatchTransformation *bt_data = data;
+
+	if (bt_data->scan == NULL) {
+		gtk_widget_destroy (bt_data->dialog);
+		g_object_unref (bt_data->gui);
+
+		if (bt_data->data->dialog == NULL)
+			dialog_data_free (bt_data->data);
+		else
+			gtk_widget_destroy (bt_data->data->dialog);
+		g_free (bt_data);
+	}
+	else
+		apply_transformation_to_all__apply_to_current (bt_data);
+}
+
+
+static void
+apply_transformation_to_all__apply_to_current (BatchTransformation *bt_data)
+{
+	FileData *file = bt_data->scan->data;
+	char     *name;
+	
+	name = basename_for_display (file->path);
+	_gtk_label_set_filename_text (GTK_LABEL (bt_data->label), name);
+	g_free (name);
+
+	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (bt_data->bar),
+				       (gdouble) (bt_data->i + 0.5) / bt_data->n);
+	
+	apply_transformation (bt_data->dialog, bt_data->data, bt_data->scan, FALSE, apply_transformation_to_all_continue, bt_data);
+	
+	bt_data->i++;	
+	bt_data->scan = bt_data->scan->next;
 }
 
 
 static void
 apply_transformation_to_all (DialogData *data)
 {
-	GladeXML  *gui;
-	GtkWidget *dialog;
-	GtkWidget *label;
-	GtkWidget *bar;
-	GList     *scan;
-	int        i, n;
+	BatchTransformation *bt_data;
 
-	gui = glade_xml_new (GTHUMB_GLADEDIR "/" PROGRESS_GLADE_FILE,
-			     NULL,
-			     NULL);
-
-	dialog = glade_xml_get_widget (gui, "progress_dialog");
-	label = glade_xml_get_widget (gui, "progress_info");
-	bar = glade_xml_get_widget (gui, "progress_progressbar");
-
-	n = g_list_length (data->current_image);
+	bt_data = g_new0 (BatchTransformation, 1);
+	bt_data->data= data;
+	bt_data->gui = glade_xml_new (GTHUMB_GLADEDIR "/" PROGRESS_GLADE_FILE,
+			     	      NULL,
+			     	      NULL);
+	bt_data->dialog = glade_xml_get_widget (bt_data->gui, "progress_dialog");
+	bt_data->label = glade_xml_get_widget (bt_data->gui, "progress_info");
+	bt_data->bar = glade_xml_get_widget (bt_data->gui, "progress_progressbar");
 
 	if (data->dialog == NULL)
-		gtk_window_set_transient_for (GTK_WINDOW (dialog),
+		gtk_window_set_transient_for (GTK_WINDOW (bt_data->dialog),
 					      GTK_WINDOW (data->window));
 	else {
 		gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
-		gtk_window_set_transient_for (GTK_WINDOW (dialog),
+		gtk_window_set_transient_for (GTK_WINDOW (bt_data->dialog),
 					      GTK_WINDOW (data->dialog));
 	}
-	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-	gtk_widget_show (dialog);
+	gtk_window_set_modal (GTK_WINDOW (bt_data->dialog), TRUE);
+	gtk_widget_show (bt_data->dialog);
 
-	while (gtk_events_pending())
-		gtk_main_iteration();
-
-	i = 0;
-	for (scan = data->current_image; scan; scan = scan->next) {
-		char *path = scan->data;
-		char *name;
-
-		name = g_filename_display_basename (path);
-		_gtk_label_set_filename_text (GTK_LABEL (label), name);
-		g_free (name);
-
-		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (bar),
-					       (gdouble) (i + 0.5) / n);
-
-		while (gtk_events_pending())
-			gtk_main_iteration();
-
-		apply_transformation (data, scan, FALSE);
-		i++;
-	}
-
-	gtk_widget_destroy (dialog);
-	g_object_unref (gui);
-
-	if (data->dialog == NULL)
-		dialog_data_free (data);
-	else
-		gtk_widget_destroy (data->dialog);
+	bt_data->n = g_list_length (data->current_image);
+	bt_data->i = 0;
+	bt_data->scan = data->current_image;
+	apply_transformation_to_all__apply_to_current (bt_data);
 }
 
 
+static void
+load_next_image_after_transformation (const char     *uri,
+				      GnomeVFSResult  result,
+				      gpointer        callback_data)
+{
+	DialogData *data = callback_data;
+	load_next_image (data);
+}  
+
+
 static void
 ok_clicked (GtkWidget  *button,
 	    DialogData *data)
@@ -389,14 +466,12 @@ ok_clicked (GtkWidget  *button,
 	gboolean to_all;
 
 	to_all = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->j_apply_to_all_checkbutton));
-
 	if (to_all) {
 		gtk_widget_hide (data->dialog);
 		apply_transformation_to_all (data);
-	} else {
-		apply_transformation (data, data->current_image, TRUE);
-		load_next_image (data);
-	}
+	} 
+	else 
+		apply_transformation (data->dialog, data, data->current_image, TRUE, load_next_image_after_transformation, data);
 }
 
 
@@ -474,6 +549,7 @@ reset_exif_tag_on_rotate_clicked (GtkWidget  *button,
 		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->j_reset_exif_tag_on_rotate_checkbutton)));
 }
 
+
 /* called when the "help" button is clicked. */
 static void
 help_cb (GtkWidget  *widget,
@@ -499,8 +575,7 @@ dlg_jpegtran (GthWindow *window)
 	GtkWidget   *reset_image;
 	GList       *list;
 
-
-	list = gth_window_get_file_list_selection (window);
+	list = gth_window_get_file_list_selection_as_fd (window);
 	if (list == NULL) {
 		g_warning ("No file selected.");
 		return;
@@ -519,7 +594,7 @@ dlg_jpegtran (GthWindow *window)
 	if (! data->gui) {
 		g_warning ("Could not find " ROTATE_GLADE_FILE "\n");
 		if (data->file_list != NULL)
-			path_list_free (data->file_list);
+			file_data_list_free (data->file_list);
 		g_free (data);
 		return;
 	}
@@ -608,7 +683,7 @@ dlg_jpegtran (GthWindow *window)
 			  data);
 
 
-	data->loader = (ImageLoader*)image_loader_new (NULL, FALSE);
+	data->loader = (ImageLoader*)image_loader_new (FALSE);
 
 	g_signal_connect (G_OBJECT (data->loader),
 			  "image_done",
@@ -638,7 +713,7 @@ dlg_apply_jpegtran (GthWindow    *window,
 	DialogData  *data;
 	GList       *list;
 
-	list = gth_window_get_file_list_selection (window);
+	list = gth_window_get_file_list_selection_as_fd (window);
 	if (list == NULL) {
 		g_warning ("No file selected.");
 		return;
