@@ -24,6 +24,8 @@
 #include <string.h>
 
 #include <glib/gi18n.h>
+#include <glib.h>
+#include <glib/gprintf.h>
 #include <gtk/gtk.h>
 #include <libgnomevfs/gnome-vfs-drive.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
@@ -61,13 +63,13 @@ enum {
 };
 
 enum {
-        CHANGED,
-	OPEN_LOCATION,
-        LAST_SIGNAL
+	CHANGED,
+	LAST_SIGNAL
 };
 
 struct _GthLocationPrivate
 {
+	gboolean               folders_only;
 	char                  *uri;
 	int                    current_idx;
 	gboolean               catalog_uri;
@@ -135,25 +137,15 @@ gth_location_class_init (GthLocationClass *class)
 	object_class->finalize = gth_location_finalize;
 
 	gth_location_signals[CHANGED] =
-                g_signal_new ("changed",
-                              G_TYPE_FROM_CLASS (class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GthLocationClass, changed),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__STRING,
-                              G_TYPE_NONE,
-                              1,
+		g_signal_new ("changed",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GthLocationClass, changed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__STRING,
+			      G_TYPE_NONE,
+			      1,
 			      G_TYPE_STRING);
-
-	gth_location_signals[OPEN_LOCATION] =
-                g_signal_new ("open_location",
-                              G_TYPE_FROM_CLASS (class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GthLocationClass, open_location),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__VOID,
-                              G_TYPE_NONE,
-                              0);
 }
 
 
@@ -163,21 +155,22 @@ gth_location_init (GthLocation *loc)
 	loc->priv = g_new0 (GthLocationPrivate, 1);
 	loc->priv->current_idx = 0;
 	loc->priv->catalog_uri = FALSE;
+	loc->priv->folders_only = FALSE;
 
 	loc->priv->volume_monitor = gnome_vfs_get_volume_monitor ();
-	g_signal_connect (loc->priv->volume_monitor, 
+	g_signal_connect (loc->priv->volume_monitor,
 			  "drive-connected",
 			  G_CALLBACK (monitor_changed_cb),
 			  loc);
-	g_signal_connect (loc->priv->volume_monitor, 
+	g_signal_connect (loc->priv->volume_monitor,
 			  "drive-disconnected",
 			  G_CALLBACK (monitor_changed_cb),
 			  loc);
-	g_signal_connect (loc->priv->volume_monitor, 
+	g_signal_connect (loc->priv->volume_monitor,
 			  "volume-mounted",
 			  G_CALLBACK (monitor_changed_cb),
 			  loc);
-	g_signal_connect (loc->priv->volume_monitor, 
+	g_signal_connect (loc->priv->volume_monitor,
 			  "volume-unmounted",
 			  G_CALLBACK (monitor_changed_cb),
 			  loc);
@@ -185,12 +178,70 @@ gth_location_init (GthLocation *loc)
 }
 
 
+static void
+open_other_location_response_cb (GtkDialog *file_sel,
+				 int        button_number,
+				 gpointer   data)
+{
+	GthLocation *location = data;
+	char        *folder;
+
+	if (button_number != GTK_RESPONSE_OK) {
+		gtk_widget_destroy (GTK_WIDGET (file_sel));
+		return;
+	}
+
+	folder = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (file_sel));
+	if (folder != NULL) {
+		gth_location_set_folder_uri (location, folder, FALSE);
+		g_signal_emit (G_OBJECT (location), gth_location_signals[CHANGED], 0, folder);
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (file_sel));
+}
+
+
+G_CONST_RETURN static char*
+gth_location_get_folder_uri (GthLocation *loc)
+{
+	if (uri_scheme_is_catalog (loc->priv->uri) || (uri_scheme_is_search (loc->priv->uri)))
+		return get_home_uri();
+	else
+		return loc->priv->uri;
+}
+
+
+void
+gth_location_open_other (GthLocation *location)
+{
+	GtkWidget *chooser;
+
+	chooser = gtk_file_chooser_dialog_new (_("Open Location"),
+					       GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (location))),
+					       GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					       GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+					       NULL);
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (chooser), FALSE);
+	gtk_window_set_modal (GTK_WINDOW (chooser), TRUE);
+	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (chooser), gth_location_get_folder_uri (location));
+
+	g_signal_connect (G_OBJECT (GTK_DIALOG (chooser)),
+			  "response",
+			  G_CALLBACK (open_other_location_response_cb),
+			  location);
+
+	gtk_window_set_modal (GTK_WINDOW (chooser), TRUE);
+	gtk_widget_show (chooser);
+}
+
+
 static void combo_changed_cb (GtkComboBox *widget,
 			      gpointer     user_data);
 
 
 static void
-reset_active_index (GthLocation *loc) 
+reset_active_index (GthLocation *loc)
 {
 	g_signal_handlers_block_by_func (loc->priv->combo, combo_changed_cb, loc);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (loc->priv->combo), loc->priv->current_idx);
@@ -209,7 +260,7 @@ combo_changed_cb (GtkComboBox *widget,
 
 	if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (loc->priv->combo), &iter))
 		return;
-	
+
 	gtk_tree_model_get (GTK_TREE_MODEL (loc->priv->model),
 			    &iter,
 			    PATH_COLUMN, &path,
@@ -218,13 +269,13 @@ combo_changed_cb (GtkComboBox *widget,
 
 	if (item_type == ITEM_TYPE_OPEN_LOCATION) {
 		reset_active_index (loc);
-		g_signal_emit (G_OBJECT (loc), gth_location_signals[OPEN_LOCATION], 0);
+		gth_location_open_other (loc);
 		return;
 	}
 
 	loc->priv->current_idx = gtk_combo_box_get_active (GTK_COMBO_BOX (loc->priv->combo));
 
-	if (path == NULL) 
+	if (path == NULL)
 		return;
 
 	g_signal_emit (G_OBJECT (loc), gth_location_signals[CHANGED], 0, path);
@@ -242,13 +293,14 @@ row_separator_func (GtkTreeModel *model,
 			    iter,
 			    TYPE_COLUMN, &item_type,
 			    -1);
-	return ((item_type == ITEM_TYPE_SEPARATOR) 
+	return ((item_type == ITEM_TYPE_SEPARATOR)
 		|| (item_type == ITEM_TYPE_BOOKMARK_SEPARATOR));
 }
 
 
 static void
-gth_location_construct (GthLocation *loc)
+gth_location_construct (GthLocation *loc,
+			gboolean     folders_only)
 {
 	GtkCellRenderer *renderer;
 	GValue           value = { 0, };
@@ -256,9 +308,10 @@ gth_location_construct (GthLocation *loc)
 	int              icon_size;
 	GdkPixbuf       *icon;
 
-	loc->priv->model = gtk_list_store_new (N_COLUMNS, 
-					       GDK_TYPE_PIXBUF, 
-					       G_TYPE_STRING, 
+	loc->priv->folders_only = folders_only;
+	loc->priv->model = gtk_list_store_new (N_COLUMNS,
+					       GDK_TYPE_PIXBUF,
+					       G_TYPE_STRING,
 					       G_TYPE_STRING,
 					       G_TYPE_INT);
 	loc->priv->combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (loc->priv->model));
@@ -307,14 +360,14 @@ gth_location_construct (GthLocation *loc)
 	/* Add standard items. */
 
 	/* separator #1 */
-	
+
 	gtk_list_store_append (loc->priv->model, &iter);
 	gtk_list_store_set (loc->priv->model, &iter,
 			    TYPE_COLUMN, ITEM_TYPE_SEPARATOR,
 			    -1);
 
 	/* separator #2 */
-	
+
 	gtk_list_store_append (loc->priv->model, &iter);
 	gtk_list_store_set (loc->priv->model, &iter,
 			    TYPE_COLUMN, ITEM_TYPE_SEPARATOR,
@@ -341,10 +394,10 @@ gth_location_construct (GthLocation *loc)
 GType
 gth_location_get_type ()
 {
-        static GType type = 0;
+	static GType type = 0;
 
-        if (! type) {
-                GTypeInfo type_info = {
+	if (! type) {
+		GTypeInfo type_info = {
 			sizeof (GthLocationClass),
 			NULL,
 			NULL,
@@ -362,17 +415,17 @@ gth_location_get_type ()
 					       0);
 	}
 
-        return type;
+	return type;
 }
 
 
-GtkWidget*     
-gth_location_new (void)
+GtkWidget*
+gth_location_new (gboolean folders_only)
 {
 	GtkWidget *widget;
 
 	widget = GTK_WIDGET (g_object_new (GTH_TYPE_LOCATION, NULL));
-	gth_location_construct (GTH_LOCATION (widget));
+	gth_location_construct (GTH_LOCATION (widget), folders_only);
 
 	return widget;
 }
@@ -391,7 +444,7 @@ get_item_from_uri (GthLocation *loc,
 
 	if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (loc->priv->model), iter))
 		return FALSE;
-	
+
 	do {
 		char *path = NULL;
 		int   item_type = ITEM_TYPE_NONE;
@@ -412,7 +465,7 @@ get_item_from_uri (GthLocation *loc,
 		if (idx != NULL)
 			*idx = *idx + 1;
 	} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (loc->priv->model), iter));
-	
+
 	return found;
 }
 
@@ -435,7 +488,7 @@ clear_items (GthLocation *loc,
 				    PATH_COLUMN, &uri,
 				    -1);
 
-		if (item_type == clear_type) 
+		if (item_type == clear_type)
 			valid = gtk_list_store_remove (loc->priv->model, &iter);
 		else
 			valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (loc->priv->model), &iter);
@@ -457,7 +510,7 @@ get_first_separator_pos (GthLocation *loc,
 
 	if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (loc->priv->model), &iter))
 		return FALSE;
-	
+
 	do {
 		int item_type = ITEM_TYPE_NONE;
 
@@ -473,7 +526,7 @@ get_first_separator_pos (GthLocation *loc,
 		if (idx != NULL)
 			*idx = *idx + 1;
 	} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (loc->priv->model), &iter));
-	
+
 	return found;
 }
 
@@ -580,9 +633,6 @@ update_drives (GthLocation *loc)
 
 	clear_items (loc, ITEM_TYPE_DRIVE);
 
-	if (g_list_length (loc->priv->drives) == 0)
-		return;
-
 	if (!get_first_separator_pos (loc, &pos))
 		return;
 
@@ -591,11 +641,15 @@ update_drives (GthLocation *loc)
 	/* Home, File System */
 
 	insert_drive_from_uri (loc, get_home_uri (), pos++);
-	insert_drive_from_uri (loc, "catalog://", pos++);
 	insert_drive_from_uri (loc, "file://", pos++);
+	if (!loc->priv->folders_only)
+		insert_drive_from_uri (loc, "catalog:///", pos++);
 
 	/* Other drives */
 
+	if (g_list_length (loc->priv->drives) == 0)
+		return;
+
 	for (scan = loc->priv->drives; scan; scan = scan->next) {
 		GnomeVFSDrive *drive = scan->data;
 		char          *uri;
@@ -671,11 +725,13 @@ get_volume_from_uri (GthLocation *loc,
 		     const char  *from_uri)
 {
 	GnomeVFSVolume *volume = NULL;
+	char           *base_uri;
 	char           *uri;
 
 	if (from_uri == NULL)
 		return NULL;
 
+	base_uri = get_base_uri (from_uri);
 	uri = g_strdup (from_uri);
 
 	while ((uri != NULL) && (volume == NULL)) {
@@ -685,18 +741,19 @@ get_volume_from_uri (GthLocation *loc,
 			g_free (uri);
 			uri = parent;
 		}
-		if (str_ends_with (uri, "://"))
+		if (same_uri (uri, base_uri))
 			uri = NULL;
 	}
 
 	g_free (uri);
+	g_free (base_uri);
 
 	return volume;
 }
 
 
 static void
-update_uri (GthLocation *loc, 
+update_uri (GthLocation *loc,
 	    gboolean     reset_history)
 {
 	GtkTreeIter     iter;
@@ -723,16 +780,18 @@ update_uri (GthLocation *loc,
 	home_uri = g_strconcat ("file://", g_get_home_dir(), NULL);
 	uri = g_strdup (loc->priv->uri);
 
-	if (loc->priv->catalog_uri)
+	if (loc->priv->catalog_uri) {
 		base_uri = g_strconcat (CATALOG_PREFIX,
 					g_get_home_dir(),
 					"/",
 					RC_CATALOG_DIR,
 					NULL);
+	}
 	else {
 		volume = get_volume_from_uri (loc, uri);
-		if (volume == NULL) 
-			base_uri = g_strdup ("file://");
+		if (volume == NULL) {
+			base_uri = get_base_uri (uri);
+		}
 		else {
 			base_uri = gnome_vfs_volume_get_activation_uri (volume);
 			drive = gnome_vfs_volume_get_drive (volume);
@@ -742,29 +801,36 @@ update_uri (GthLocation *loc,
 	while (uri != NULL) {
 		char *parent;
 
-		if (loc->priv->catalog_uri) 
+		if (loc->priv->catalog_uri)
 			pixbuf = gdk_pixbuf_new_from_inline (-1, library_19_rgba, FALSE, NULL);
 		else {
-			if (same_uri (uri, base_uri)) 
+			if (same_uri (uri, base_uri))
 				pixbuf = get_drive_icon (loc, drive);
 			else
 				pixbuf = get_icon_for_uri (GTK_WIDGET (loc), uri);
 		}
 
 		if (same_uri (uri, base_uri)) {
-			if (loc->priv->catalog_uri)
+			if (loc->priv->catalog_uri) {
 				uri_name = g_strdup (_("Catalogs"));
-			else {
-				if (volume != NULL)
-					uri_name = gnome_vfs_volume_get_display_name (volume);
-				else
-					uri_name = g_strdup (_("File System"));
 			}
-		} else {
+			else {
+				if (volume != NULL) {
+					uri_name = gnome_vfs_volume_get_display_name (volume);
+				}
+				else {
+					if (uri_scheme_is_file (base_uri))
+						uri_name = g_strdup (_("File System"));
+					else
+						uri_name = g_strdup (base_uri);
+				}
+			}
+		} 
+		else {
 			if (same_uri (uri, home_uri))
 				uri_name = g_strdup (_("Home"));
 			else
-				uri_name = g_filename_display_basename (uri);
+				uri_name = gnome_vfs_unescape_string_for_display (file_name_from_path (uri));
 		}
 
 		gtk_list_store_insert (loc->priv->model, &iter, pos++);
@@ -807,7 +873,7 @@ gth_location_set_uri (GthLocation *loc,
 		      gboolean     reset_history)
 {
 	g_free (loc->priv->uri);
-	loc->priv->uri = get_uri_from_path (uri);
+	loc->priv->uri = add_scheme_if_absent (uri);
 	update_uri (loc, reset_history);
 }
 
@@ -819,7 +885,7 @@ gth_location_set_folder_uri (GthLocation *loc,
 {
 	loc->priv->catalog_uri = FALSE;
 	gth_location_set_uri (loc, uri, reset_history);
-	
+
 }
 
 
@@ -833,7 +899,7 @@ gth_location_set_catalog_uri (GthLocation *loc,
 	loc->priv->catalog_uri = TRUE;
 
 	if (! uri_scheme_is_catalog (uri))
-		catalog_uri = g_strconcat (CATALOG_PREFIX, remove_scheme_from_uri (uri), NULL);
+		catalog_uri = g_strconcat (CATALOG_PREFIX, remove_host_from_uri (uri), NULL);
 	else
 		catalog_uri = g_strdup (uri);
 	gth_location_set_uri (loc, catalog_uri, reset_history);
@@ -860,6 +926,13 @@ gth_location_set_bookmarks (GthLocation *loc,
 	clear_items (loc, ITEM_TYPE_BOOKMARK);
 	clear_items (loc, ITEM_TYPE_BOOKMARK_SEPARATOR);
 
+	if (max_size < 0) {
+		if (bookmark_list != NULL)
+			max_size = g_list_length (bookmark_list);
+		else
+			max_size = 0;
+	}
+
 	if (max_size == 0)
 		return;
 
@@ -877,6 +950,9 @@ gth_location_set_bookmarks (GthLocation *loc,
 		char       *uri_name;
 		GdkPixbuf  *pixbuf;
 
+		if (loc->priv->folders_only && (uri_scheme_is_catalog (uri) || uri_scheme_is_search (uri)))
+			continue;
+
 		uri_name = get_uri_display_name (uri);
 
 		pixbuf = get_icon_for_uri (GTK_WIDGET (loc), uri);
